package ctbrec.recorder.download.hls;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.Config;
import ctbrec.OS;
import ctbrec.io.HttpClient;
import ctbrec.io.StreamRedirectThread;

public class MergedHlsDownload extends HlsDownload {
    private static final Logger LOG = LoggerFactory.getLogger(MergedHlsDownload.class);

    public MergedHlsDownload(HttpClient client) {
        super(client);
    }

    @Override
    public void postprocess(ctbrec.Recording recording) {
        super.postprocess(recording);
        File dir = new File(Config.getInstance().getSettings().recordingsDir, recording.getPath());
        File playlist = new File(dir, "playlist.m3u8");
        if (!playlist.exists()) {
            super.generatePlaylist(recording);
        }

        try {
            // @formatter:off
            String[] cmdline = OS.getFFmpegCommand(
                    "-i", playlist.getAbsolutePath(),
                    "-c:v", "copy",
                    "-c:a", "copy",
                    "-movflags", "faststart",
                    "-f", "mp4",
                    new File(dir, "0merged.mp4").getAbsolutePath()
                    );
            // @formatter:on
            LOG.debug("Command line: {}", Arrays.toString(cmdline));
            Process ffmpeg = Runtime.getRuntime().exec(cmdline);
            new Thread(new StreamRedirectThread(ffmpeg.getInputStream(), System.out)).start(); // NOSONAR
            new Thread(new StreamRedirectThread(ffmpeg.getErrorStream(), System.err)).start(); // NOSONAR
            int exitCode = ffmpeg.waitFor();
            if (exitCode == 0) {
                recording.setPath(recording.getPath() + '/' + "0merged.mp4");
                Files.delete(playlist.toPath());
                File[] segments = dir.listFiles((directory, filename) -> filename.endsWith(".ts"));
                for (File segment : segments) {
                    Files.delete(segment.toPath());
                }
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            LOG.error("Interrupted while waiting for FFMPEG", e);
        } catch (IOException e) {
            LOG.error("Couldn't execute FFMPEG", e);
        }
    }
}
